(function () {
  const defaultBoxShape = {
    h: 100,
    w: 100,
    x: 100,
    y: 0,
    hr: true,
    wr: false,
    xr: false,
    yr: false,
    move: true
  }

  const defaultOffset = {
    x: 0,
    y: 0
  }

  const defaultParentShape = {
    h: 100,
    w: 100,
    x: 0,
    y: 0
  }

  const uuid = () => {
    return ([1e7] + -1e3 + -4e3 + -8e3 + -1e11).replace(/[018]/g, c =>
      (c ^ crypto.getRandomValues(new Uint8Array(1))[0] & 15 >> c / 4).toString(16)
    );
  }

  class AswDragResize {

    constructor(element, parentElement = document.body, { boxShape, events = {} }) {
      const makeActive = (e) => {
        e.preventDefault()
        e.stopPropagation()
        if (boxShape.active) {
          setParentShape()
          mask = document.getElementById(this.id)
          if (!mask) {
            mask = document.createElement('div')
            mask.setAttribute('class', 'asw-dr-mask')
            mask.setAttribute('id', this.id)
            mask.innerHTML = `<div class="mask-container">
            <div class="resizer top-left" value="tl" ></div>
            <div class="resizer top-right" value="tr" ></div>
            <div class="resizer bottom-left" value="bl" ></div>
            <div class="resizer bottom-right" value="br" ></div>
            <div class="moving-mask" id="moving_mask" tabindex="1"></div>
          </div>`
            document.body.append(mask)
          } else {
            mask.style.display = 'block'
          }
          changeShape(boxShape)
          active_drag()
          active_resize()
          window.addEventListener('click', checkClickOutside)
          events.onActive && events.onActive(boxShape)
        }
      }
  
      const checkClickOutside = (e) => {
        if (!mask.contains(e.target)) {
          window.removeEventListener('click', checkClickOutside)
          inactive_drag()
          inactive_resize()
          mask.style.display = 'none'
          events.onInactive && events.onInactive(boxShape)
        }
      }
  
      const changeShape = (boxShape, withMask = true) => {
  
        element.style.top = boxShape.y + (boxShape.yr ? '%' : 'px')
        element.style.left = boxShape.x + (boxShape.xr ? '%' : 'px')
        element.style.height = boxShape.h + (boxShape.hr ? '%' : 'px')
        element.style.width = boxShape.w + (boxShape.wr ? '%' : 'px')
        if (withMask) {
          const elementBondingBox = element.getBoundingClientRect()
          mask.style.top = elementBondingBox.top + 'px'
          mask.style.left = elementBondingBox.left + 'px'
          mask.style.width = elementBondingBox.width + 'px'
          mask.style.height = elementBondingBox.height + 'px'
        }
      }
  
      const setParentShape = () => {
        const tempParentShape = parentElement.getBoundingClientRect()
        parentShape.h = tempParentShape.height
        parentShape.w = tempParentShape.width
        parentShape.x = tempParentShape.x
        parentShape.y = tempParentShape.y
      }
  
      const setOffset = (e) => {
        offset.x = e.offsetX
        offset.y = e.offsetY
      }
  
      const per_to_px = () => {
        boxShape.x = boxShape.xr ? ((parentShape.w * boxShape.x) / 100) : boxShape.x
        boxShape.y = boxShape.yr ? ((parentShape.h * boxShape.y) / 100) : boxShape.y
        boxShape.w = boxShape.wr ? ((parentShape.w * boxShape.w) / 100) : boxShape.w
        boxShape.h = boxShape.hr ? ((parentShape.h * boxShape.h) / 100) : boxShape.h
      }
  
      const px_to_per = () => {
        boxShape.x = boxShape.xr ? (boxShape.x * 100) / parentShape.w : boxShape.x
        boxShape.y = boxShape.yr ? (boxShape.y * 100) / parentShape.h : boxShape.y
        boxShape.w = boxShape.wr ? (boxShape.w * 100) / parentShape.w : boxShape.w
        boxShape.h = boxShape.hr ? (boxShape.h * 100) / parentShape.h : boxShape.h
      }
  
      const active_drag = () => {
        movingMask = mask.querySelector('#moving_mask')
        movingMask.addEventListener('mousedown', start_drag)
        window.addEventListener('mouseup', stop_drag)
        events.onDragActive && events.onDragActive(boxShape)
      }
  
      const inactive_drag = () => {
        movingMask.removeEventListener('mousedown', start_drag)
        window.removeEventListener('mouseup', stop_drag)
        events.onDragInactive && events.onDragInactive(boxShape)
      }
  
      const start_drag = (e) => {
        e.preventDefault()
        setParentShape()
        setOffset(e)
        window.addEventListener('mousemove', draging)
        events.onDragStart && events.onDragStart(boxShape)
      }
  
      const stop_drag = (e) => {
        e.preventDefault()
        window.removeEventListener('mousemove', draging)
        events.onDragStop && events.onDragStop(boxShape)
  
      }
  
      const draging = (e) => {
        e.preventDefault()
        const x = e.pageX
        const y = e.pageY
  
        per_to_px()
  
        boxShape.x = x - parentShape.x - offset.x
        boxShape.x = boxShape.x < 0 ? 0 : boxShape.x
        boxShape.x = boxShape.x > parentShape.w - boxShape.w ? parentShape.w - boxShape.w : boxShape.x
  
        boxShape.y = y - parentShape.y - offset.y
        boxShape.y = boxShape.y < 0 ? 0 : boxShape.y
        boxShape.y = boxShape.y > parentShape.h - boxShape.h ? parentShape.h - boxShape.h : boxShape.y
  
        px_to_per()
        changeShape(boxShape)
        events.onDraging && events.onDraging(boxShape)
      }
  
      const active_resize = () => {
        resize_selectors = mask.querySelectorAll('.resizer') || []
        resize_selectors.forEach(resize => resize.addEventListener('mousedown', start_resize))
        window.addEventListener('mouseup', stop_resize)
        events.onResizeActive && events.onResizeActive(boxShape)
      }
  
      const inactive_resize = () => {
        resize_selectors = mask.querySelectorAll('.resizer') || []
        resize_selectors.forEach(resize => resize.removeEventListener('mousedown', start_resize))
        window.removeEventListener('mouseup', stop_resize)
        events.onResizeInactive && events.onResizeInactive(boxShape)
      }
  
      const start_resize = (e) => {
        e.preventDefault()
        active_resizer = e.target.getAttribute('value')
        window.addEventListener('mousemove', resizer_event_map[active_resizer])
        events.onResizeStart && events.onResizeStart(boxShape)
      }
  
      const stop_resize = (e) => {
        e.preventDefault()
        window.removeEventListener('mousemove', resizer_event_map[active_resizer])
        events.onResizeStop && events.onResizeStop(boxShape)
      }
  
      const resize_top_left = (e) => {
        const x = e.pageX
        const y = e.pageY
  
        per_to_px()
  
        const boxX = boxShape.x
        const boxY = boxShape.y
  
        boxShape.x = x - parentShape.x < 0 ? 0 : x - parentShape.x
        boxShape.y = y - parentShape.y < 0 ? 0 : y - parentShape.y
        boxShape.w = boxShape.w + boxX - boxShape.x
        boxShape.h = boxShape.h + boxY - boxShape.y
  
        px_to_per()
        changeShape(boxShape)
        events.onResizeing && events.onResizeing(boxShape, 'tl')
      }
  
      const resize_top_right = (e) => {
        const x = e.pageX
        const y = e.pageY
  
        per_to_px()
  
        const boxX = boxShape.x
        const boxY = boxShape.y
  
        boxShape.y = y - parentShape.y < 0 ? 0 : y - parentShape.y
        boxShape.h = boxShape.h + boxY - boxShape.y
        boxShape.w = x - boxX - parentShape.x
        boxShape.w = boxShape.w + boxShape.x > parentShape.w ? parentShape.w - boxShape.x : boxShape.w
  
  
        px_to_per()
        changeShape(boxShape)
        events.onResizeing && events.onResizeing(boxShape, 'tr')
      }
  
      const resize_bottom_right = (e) => {
        const x = e.pageX
        const y = e.pageY
  
        per_to_px()
  
        const boxX = boxShape.x
        const boxY = boxShape.y
  
        boxShape.w = x - boxX - parentShape.x
        boxShape.w = boxShape.w + boxShape.x > parentShape.w ? parentShape.w - boxShape.x : boxShape.w
        boxShape.h = y - boxY - parentShape.y
        boxShape.h = boxShape.h + boxShape.y > parentShape.h ? parentShape.h - boxShape.y : boxShape.h
  
  
        px_to_per()
        changeShape(boxShape)
        events.onResizeing && events.onResizeing(boxShape, 'br')
      }
  
      const resize_bottom_left = (e) => {
        const x = e.pageX
        const y = e.pageY
  
        per_to_px()
  
        const boxX = boxShape.x
        const boxY = boxShape.y
  
        boxShape.x = x - parentShape.x < 0 ? 0 : x - parentShape.x
        boxShape.w = boxShape.w + boxX - boxShape.x
        boxShape.h = y - boxY - parentShape.y
        boxShape.h = boxShape.h + boxShape.y > parentShape.h ? parentShape.h - boxShape.y : boxShape.h
  
        px_to_per()
        changeShape(boxShape)
        events.onResizeing && events.onResizeing(boxShape, 'bl')
      }
  
      boxShape = boxShape || { ...defaultBoxShape }
      parentElement = parentElement || document.body
      parentElement.style.position = 'relative'
      element.style.position = 'absolute'
      let parentShape = { ...defaultParentShape }
      let offset = { ...defaultOffset }
      let movingMask = null
      let resize_selectors = null
      let resizer_event_map = {
        tl: resize_top_left,
        tr: resize_top_right,
        br: resize_bottom_right,
        bl: resize_bottom_left
      }
      let active_resizer = null
      let mask =
        this.id = 'mask-' + uuid()
      boxShape.active && element.addEventListener('click', makeActive)
      changeShape(boxShape, false)
  
      this.setBoxShape = (newShape) => {
        changeShape({ ...boxShape, ...newShape })
      }
  
      this.active = () => {
        element.addEventListener('click', makeActive)
      }
  
      this.inactive = () => {
        element.removeEventListener('click', makeActive)
      }
    }
  }
  if (window && document) {
    if (!window.AswDragResize) {
      window.AswDragResize = AswDragResize
    }
  } else {
    console.error('window and document object are not defind, make sure you have enable javascript support to the browser')
  }
})()
